const linkWrapperPrefBranch = "link_wrapper.";
const linkWrapperEnabledPref = linkWrapperPrefBranch + "enabled";
const linkWrapperShowStatusBarPref = linkWrapperPrefBranch + "showStatusBar";
const linkWrapperToolsMenuPref = linkWrapperPrefBranch + "showToolsMenu";

var prefBranch_linkWrapper = null;

var link_wrapper = {

  init: function() {
    
    link_wrapper.register();

    link_wrapper.resetValues();

    link_wrapper.setForceReload("false");

    try {
      document.getElementById("linkWrapper-status-deck").hidden = !nsPreferences.getBoolPref(linkWrapperShowStatusBarPref)
    } catch (ex) {
      //to_console(ex);
    }
    
    try {
      document.getElementById("linkWrapperToolsOptionsMenu").hidden = !nsPreferences.getBoolPref(linkWrapperToolsMenuPref)
    } catch (ex) {
      //to_console(ex);
    }
    
    var appcontent = document.getElementById("appcontent");   // browser

    if(appcontent) {
      appcontent.addEventListener("DOMContentLoaded", link_wrapper.linkParser, true);
      //appcontent.addEventListener("DOMTitleChanged", link_wrapper.resetOnPageChange, false);
    }

    gBrowser.mTabContainer.addEventListener("DOMNodeInserted", link_wrapper.resetTabSettings, true);

    gBrowser.mTabContainer.addEventListener("select", link_wrapper.switchTabs, true);

    return true;
  },

  linkParser: function(e) {
    var linkWrapperEnabled = nsPreferences.getBoolPref(linkWrapperEnabledPref)
    var forceReload = link_wrapper.getForceReload();

    var ndDocument = e.originalTarget;
    var aBodyTags = ndDocument.getElementsByTagName("body");
    var ndBody = (aBodyTags.length > 0) ? aBodyTags[0] : false;
    
    link_wrapper.setStatus(ndBody, linkWrapperEnabled);

    //  wrapping disabled and not forcing a reload
    if (!linkWrapperEnabled && forceReload == "false") {
      link_wrapper.showStatus();
      return false;
    }
    
    // wrapping enabled, but users want to see unwrapped by clicking statusbar
    if (linkWrapperEnabled && forceReload == "bypass") {
      link_wrapper.setStatus(ndBody, "false");
      link_wrapper.showStatus();
      return false;
    }
    
    var win = e.explicitOriginalTarget.defaultView
    var sIgnoreList = nsPreferences.copyUnicharPref(linkWrapperPrefBranch +"ignoreList");

    if (win.location && win.location.hostname && sIgnoreList) {
      var regEx_IgnoreList = new RegExp(sIgnoreList, "ig")
      try {
        if (regEx_IgnoreList.test(win.location.hostname)) {
          if (forceReload != "false") {
            link_wrapper.setStatus(ndBody, "true");
          } else {
            link_wrapper.setStatus(ndBody, "blacklist");
          }
          link_wrapper.showStatus();
          if (forceReload == "false") {
            return false;
          }
        }
      } catch(ex) {
        to_console(ex);
        return false;
      }
    }

    try {
      if (nsPreferences.getBoolPref(linkWrapperPrefBranch + "charLimitEnabled")) {
        var objRange = document.createRange();
        objRange.setEnd(ndBody, ndBody.childNodes.length);
        objRange.setStart(ndBody, 0);
  
        var nPageLength = objRange.toString().length;
        objRange.detach();
  
        if (nPageLength > nsPreferences.getIntPref(linkWrapperPrefBranch + "charLimit")) {
          if (forceReload != "false") {
            link_wrapper.setStatus(ndBody, "true");
          } else {
            link_wrapper.setStatus(ndBody, "limit");
          }
          link_wrapper.showStatus();
          if (forceReload == "false") {
            return false;
          }
        }
      }
    } catch (ex) {
      to_console(ex);
      return false;
    }

    var bModFlag = false;
    var winDoc = win.document;
    var regEx_IgnoreTags = new RegExp(nsPreferences.copyUnicharPref(linkWrapperPrefBranch + "ignoreTags"), "ig")

    var wrapLength = nsPreferences.getIntPref(linkWrapperPrefBranch + "wrapLength");
    var regEx_Wrapper = "\\S{" + wrapLength + "}";

    function line_parser(n) {
      var u, r, c, x;

      if (n.nodeType == 3) {
        u = n.data.search(regEx_Wrapper);
//          to_console(n.data + " : " + n.nodeType);

        if (u >= 0) {
          r = n.splitText(u + wrapLength);
          n.parentNode.insertBefore(winDoc.createElement('wbr'), r);
          bModFlag = true;
        }
      } else if (!regEx_IgnoreTags.test(n.tagName)) {
        for (c = 0; x = n.childNodes[c]; ++c) {
//          to_console(n.tagName);
          line_parser(x);
        }
      }
    }

    if (winDoc.body) {
      line_parser(winDoc.body);
    }

    if (bModFlag) {
      try {
        var headElem = window.content.document.getElementsByTagName("head")[0];
        if (!headElem) {
          return false;
        }

        elem = window.content.document.createElement("style");
        elem.id = "toggleprewrap";
        elem.type = "text/css";
        elem.innerHTML = "pre { white-space: -moz-pre-wrap !important; } /* inserted by the Link Wrapper extension */";
        headElem.appendChild(elem);

        // Coded added to fix regression is Firefox 2.0/3.0 dev builds
        var thisE = winDoc.body
        var p = thisE.parentNode;
        var n = thisE.nextSibling;
        p.removeChild(thisE);
        p.insertBefore(thisE, n);
    } catch (ex) {
        to_console(ex);
      }
    }

    if (forceReload == "false") {
      link_wrapper.showStatus();
    }

    return true;
  },
  
  toogleStatus: function() {
    var linkWrapperStatusImage = document.getElementById('linkWrapper-status-bar-image');
    var linkWrapperStatus = linkWrapperStatusImage.getAttribute("status");

    var forceValue = link_wrapper.getForceReload();

    switch (linkWrapperStatus) {
      case "limit":
      case "blacklist":
        forceValue = (forceValue != "true") ? "true" : "false";
        break;
      default:
        forceValue = (forceValue != "bypass") ? "bypass" : "false";
        break;
    }

    link_wrapper.setForceReload(forceValue)
    link_wrapper.showStatus();

    gBrowser.reload();
    //BrowserReload();
  },

  register: function() {
    prefBranch_linkWrapper = link_wrapper.getPrefBranch();

    var pbi = linkWrapper_common.getBranchPrefInterface(prefBranch_linkWrapper);
    pbi.addObserver("", this, false);
  },

  unregister: function() {

    var appcontent = document.getElementById("appcontent");   // browser

    if(appcontent) {
      appcontent.removeEventListener("DOMContentLoaded", link_wrapper.linkParser, true);
      //appcontent.addEventListener("DOMTitleChanged", link_wrapper.resetOnPageChange, false);
    }

    gBrowser.mTabContainer.removeEventListener("DOMNodeInserted", link_wrapper.resetTabSettings, true);

    gBrowser.mTabContainer.removeEventListener("select", link_wrapper.switchTabs, true);


    if(!prefBranch_linkWrapper) return;

    var pbi = linkWrapper_common.getBranchPrefInterface(prefBranch_linkWrapper);
    try {
      pbi.removeObserver("", this);
    } catch(ex) {
      to_console(ex);
    }
  },

  observe: function(aSubject, aTopic, aData) {
    if (aTopic != "nsPref:changed") return;

    switch (aData) {
      case "showToolsMenu":
        document.getElementById("linkWrapperToolsOptionsMenu").hidden = !nsPreferences.getBoolPref(linkWrapperToolsMenuPref)
        break;
      
      case "showStatusBar":
        document.getElementById("linkWrapper-status-deck").hidden = !nsPreferences.getBoolPref(linkWrapperShowStatusBarPref)
        break;

      case "enabled":
        var linkWrapperEnabled = nsPreferences.getBoolPref(linkWrapperEnabledPref)
        var appcontent = document.getElementById("appcontent");   // browser
    
        if (linkWrapperEnabled && appcontent) {
          appcontent.addEventListener("DOMContentLoaded", link_wrapper.linkParser, false);
          gBrowser.mTabContainer.addEventListener("select", link_wrapper.showStatus, true);
        } else {
          appcontent.removeEventListener("DOMContentLoaded", link_wrapper.linkParser, false);
          gBrowser.mTabContainer.removeEventListener("select", link_wrapper.showStatus, true);
        }

        gBrowser.reloadAllTabs();
        //BrowserReload();
        break;
    }
  },
  
  getPrefBranch: function() {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].
    getService(Components.interfaces.nsIPrefService);

    return prefService.getBranch(linkWrapperPrefBranch);
  },
  
  setStatus: function(ndBody, currentStatus) {
    try {
      ndBody.setAttribute("linkWrapperStatus", currentStatus);
    } catch (ex) {
      to_console(ex);
    }
  },  

  showStatus: function() {

//    var ndDocument = gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument;
    var ndDocument = gBrowser.getBrowserForTab(gBrowser.mCurrentTab).contentDocument;
    var aBodyTags = ndDocument.getElementsByTagName("body");
    var ndBody = (aBodyTags.length > 0) ? aBodyTags[0] : false;
    
    if (!ndBody) {
      return false;
    }

    var currentStatus = ndBody.getAttribute("linkWrapperStatus");
  
    var linkWrapperStatusImage = document.getElementById('linkWrapper-status-bar-image');
    linkWrapperStatusImage.setAttribute("status", currentStatus);

    return true;

  },
  
  resetTabSettings: function(e) {
    try {
      if (e.target.localName == "tab") {
        var linkWrapperEnabled = nsPreferences.getBoolPref(linkWrapperEnabledPref)
        
        var ndDocument = e.originalTarget;
        var aBodyTags = ndDocument.getElementsByTagName("body");
        var ndBody = (aBodyTags.length > 0) ? aBodyTags[0] : false;
        
        if (!ndBody) {
          return false;
        }
        
        link_wrapper.setStatus(ndBody, linkWrapperEnabled);
        link_wrapper.setForceReload("false");
        link_wrapper.showStatus();
      }
    } catch (ex) {
      to_console(ex);
      return false;
    }

    return true;
  },
  
  resetOnPageChange: function() {
    if (link_wrapper.getForceReload() == "false") {
      link_wrapper.showStatus();
    }

    link_wrapper.setForceReload("false");

    return true;
  },
  
  switchTabs: function() {
    link_wrapper.setForceReload("false");
    link_wrapper.showStatus();
  },

  setForceReload: function(reloadStatus) {
    var appcontent = document.getElementById("appcontent");   // browser
    appcontent.setAttribute("forceReload", reloadStatus);

    return true;
  },  

  getForceReload: function() {
    var appcontent = document.getElementById("appcontent");   // browser
    var forceReloadStatus = appcontent.getAttribute("forceReload");

    return (!forceReloadStatus) ? "false" : forceReloadStatus;
  },  

  openSettings: function (myOptions) {
    myOptions = myOptions == null ? 'chrome,dialog=no,centerscreen' : myOptions;

    if (linkWrapper_common.noWindowTypeToFocus("link_wrapper:settings")) {
      openDialog('chrome://link_wrapper/content/settings.xul', 'link_wrapper', myOptions);
    }
  },

  resetValues: function() {

    var myResetVersion = nsPreferences.copyUnicharPref(linkWrapperPrefBranch + "resetVersion");
    var thisResetVersion = "2.0.0.0";
    
    if (myResetVersion != thisResetVersion) {
      var nsIPrefBranchObj = link_wrapper.getPrefBranch();
      var prefCount = {value:0}; 
      var prefArray = nsIPrefBranchObj.getChildList("" , prefCount); 
  
      var branchService = nsPreferences.mPrefService;
  
      for (var i = 0; i < prefArray.length; ++i) {
        if (branchService.prefHasUserValue(linkWrapperPrefBranch + prefArray[i])) {
           branchService.clearUserPref(linkWrapperPrefBranch + prefArray[i]);
        }
      } 

      nsPreferences.setUnicharPref(linkWrapperPrefBranch + "resetVersion", thisResetVersion);
    }
  },
  
  activateOptionsContext: function(event) {	
  	var linkWrapperStatusbarPanel = document.getElementById("linkWrapper-status");
  	var linkWrapperContextMenu = document.getElementById("linkWrapperContextMenu");
  	var xCoord = event.clientX;
  	var yCoord = event.clientY;
  	document.popupNode = linkWrapperStatusbarPanel;
  	linkWrapperContextMenu.showPopup(linkWrapperStatusbarPanel, xCoord, yCoord, "bottomleft", "topleft");
  }
}

/* used for debugging only */
var mConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService)

function to_console(msg) {
  mConsoleService.logStringMessage("link_wrapper: " + msg);
}


//window.document.getElementById("appcontent").addEventListener("DOMContentLoaded", link_wrapper.linkParser, false);
// from http://kb.mozillazine.org/On_page_load
window.addEventListener("load", link_wrapper.init, false);
window.addEventListener("unload", link_wrapper.unregister, false);
